#ifndef GDBM_POOL_H
#define GDBM_POOL_H
#include <string>
#include "gdbm.h"
 
class gdbm_pool
{
  private:
    typedef unsigned long long intkeytype;
    //typedef char charkeytype[16];
    typedef unsigned long long charkeytype;
    const static int keylen = sizeof(charkeytype);  // including trailing zero
    const static int maxvallen = 1024;  // including trailing zero
    char* file_name;        // gdmb file name
    std::string value;      // value of requested record
    charkeytype key;        // record to search or insert
    std::string statuskey;  // record to store the status
    std::string nextkey;    // key for next datarecord
    std::string lastkey;    // key for getting last datakey
    std::string firstkey;   // key for getting first key;
    charkeytype zerokey;    // key for getting zero record
    GDBM_FILE dbf;          // handle for access database
    static const int sync = GDBM_SYNC;
    //static const int sync = 0;
    struct datarecord
    {
      charkeytype prev;
      charkeytype next;
      charkeytype retrieved;
      char value[maxvallen];
    } ;
    struct zerorecord
    {
      charkeytype next;
      charkeytype first;
      charkeytype last;
      charkeytype count;
    } zero;


    //void genkey(charkeytype key,const intkeytype n);
    void genkey(charkeytype &key,const charkeytype n);


  int get_record(datarecord &r,charkeytype &k);
  int put_record(datarecord &r,charkeytype &k);
  int get_zero(void);
  int put_zero(void);


  public:
  // constructor ()
  gdbm_pool();

  // destructor

  ~gdbm_pool();

  void set_file_name(std::string filename);
  std::string get_file_name(void);

  int create_db(void);
  int add_line(std::string line);
  int get_line(std::string &line);

  int open_db(char rw);
  int close_db(void);
    const static int NOFILENAME  = 1;
    const static int OPENERROR   = 2;
    const static int STOREERROR  = 3;
    const static int FETCHERROR  = 4;
    const static int NOTFOUND    = 5;

};
#endif
