#ifndef STOPOS_POOL_H
#define STOPOS_POOL_H
#include <string>
#include "stopos.h"
#include "stopos_key.h"
#include <iostream>
 
class datarecord
{
  public:
    datarecord();
    datarecord(const std::string &s,char sep);
    stopos_key prev;
    stopos_key next;
    longuint retrieved;
    std::string value;
    std::string str(char dbsep) const;
};

class statusrecord
{
  public:
  stopos_key next;      // points to first line for the 'next' command
  stopos_key first;     // points to first record
  stopos_key last;      // points to last record
  stopos_key nextd;     // points to record to be dumped
  longuint count;       // total number of records added
  longuint present;     // total number of records now availabe
  longuint present0;    // ditto, but uncommitted
  std::string extra;    // string that can be used by derived class

  statusrecord(void);
  statusrecord(const std::string &s,char dbsep);
  std::string str(char dbsep);
  void init(void);
};

class stopos_pool
{
  protected:
    char* db_name;        // name for database
    std::string sdb_name; // idem
    static const unsigned int reclen = 4096;
    bool kvp; //true if Koot-Vermin paradigm is to be used
    bool db_open;
    
    stopos_key statuskey;
    stopos_key nextkey;
    statusrecord status;

    virtual int get_record(std::string &r,       const std::string &k, const std::string &l)=0;
    virtual int put_record(const std::string &r, const std::string &k, const std::string &l)=0;
    virtual int remove_record(                   const std::string &k, const std::string &l)=0;

    int put_status(void);
    int get_status(void);

    virtual std::string key_to_slot(const std::string &k) = 0;
    // the function add_line calls this to get a slot
    // from the derived class.  If no slot is necessary,
    // this function may return "".
    // The subsequent put_record call will have the return
    // value of this function as parameter.

    int get_record(datarecord &r,const stopos_key &k);
    int put_record(const datarecord &r,const stopos_key &k);
    int remove_record(const stopos_key &k);

    std::string whoami;

  public:

    stopos_pool();
    ~stopos_pool();

    virtual int create_db(const std::string &dbname) = 0;  // todo private
    virtual int purge_db(void)  = 0;
    virtual int open_db(void)   = 0;
    virtual int close_db(void)  = 0;
    virtual void dump_db(void)  = 0;


    int create_db(void);

    virtual void set_db_name(std::string filename);
    virtual std::string get_db_name(void);

    virtual int add_line(const std::string line,std::string &key);
    virtual int get_line(std::string &line,longuint &retrieved,std::string &key);
    virtual int remove_line(const std::string &key);

    virtual void set_kvp(const bool v);
    virtual bool get_kvp(void);

    int get_counts(longuint &count, 
		   longuint &present, 
		   longuint &present0);

    int dump_line(std::string  &line,
	          longuint     &retrieved,
		  std::string  &key);

    bool is_statusrecord(const std::string &k);

    const static int NOFILENAME        =  1;
    const static int OPENERROR         =  2;
    const static int STOREERROR        =  3;
    const static int FETCHERROR        =  4;
    const static int NOTFOUND          =  5;
    const static int NOTIMPLEMENTED    =  6;
    const static int REMOVEERROR       =  7;
    const static int LOCKERROR         =  8;
    const static int CLOSEERROR        =  9;
    const static int UNLOCKERROR       = 10;
    const static int DBNOTOPEN         = 11;
    const static int CREATEERROR       = 12;
    const static int PURGEERROR        = 13;
    const static int RECORDTOOLONG     = 14;

};
#endif
